﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;

namespace SGDatUnpacker
{
    class FileEntry
    {
        public Int32 Offset;
        public string Filename;
        public Int32 Length;
    }

    class Program
    {
        private static string WelcomeMessage        = "Place the .dat file in this directory, then type in the name and hit enter:";
        private static string NoFileErrorMessage    = "No file provided :(";
        private static string GoodbyeMessage        = "All unpacked! Enjoy - Jovan";

        private static string UnpackingMessageFmt   = "... Unpacking {0} ...";

        static string ClipNullStringEnding(string filename)
        {
            var terminatingChar = filename.IndexOf('\0');

            if (terminatingChar != -1)
            {
                return filename.Remove(terminatingChar);
            }

            return filename;
        }

        static void Main(string[] args)
        {
            Console.WriteLine(WelcomeMessage);
            var filename = Console.ReadLine();

            if (String.IsNullOrWhiteSpace(filename))
            {
                Console.WriteLine(NoFileErrorMessage);
                Thread.Sleep(1200);
                return;
            }

            Console.WriteLine(UnpackingMessageFmt, filename);
            
            var file = File.Open(filename, FileMode.Open, FileAccess.Read);
            var reader = new BinaryReader(file, new ASCIIEncoding());

            //
            // Parse the header.
            //
            var numFiles = reader.ReadInt32();

            var list = new List<FileEntry>();

            //
            // Read first file independently, so we can do a cleaner rolling length.
            //

            var firstEntry = new FileEntry
            {
                Offset = reader.ReadInt32(),
                Filename = ClipNullStringEnding(new string(reader.ReadChars(13)))
            };

            list.Add(firstEntry);

            for (var i = 1; i < numFiles; i++)
            {
                var entry = new FileEntry
                {
                    Offset = reader.ReadInt32(),
                    Filename = ClipNullStringEnding(new string(reader.ReadChars(13)))
                };

                list.Add(entry);

                list[i - 1].Length = entry.Offset - list[i - 1].Offset;
            }

            //
            // Last file is a fodder, take it out.
            //

            list.RemoveAt(list.Count - 1);

            //
            // Write the files out.
            //

            var directoryName = filename + ".Unpacked";
            Directory.CreateDirectory(directoryName);

            foreach (var entry in list)
            {
                var newFile = File.OpenWrite(Path.Combine(directoryName, entry.Filename.Replace("\0", String.Empty)));
                file.Seek(entry.Offset, SeekOrigin.Begin);

                newFile.Write(reader.ReadBytes(entry.Length), 0, entry.Length);

                newFile.Close();
            }

            reader.Close();
            file.Close();

            Console.WriteLine(GoodbyeMessage);
            Thread.Sleep(1200);
        }
    }
}
